<?php

/**
 * @file
 * embridge.admin.inc
 * Defines admin config. screens and functionality for the embridge module
 */

/**
 * General admin settings form for embridge.
 */
function embridge_settings() {

  // Add JS for entermedia.
  drupal_add_js(drupal_get_path('module', 'embridge') . '/js/embridge.js');
  drupal_add_css(drupal_get_path('module', 'embridge') . '/css/embridge.css');

  $form = array();

  // Server information.
  $form['server_info'] = array(
    '#type' => 'fieldset',
    '#title' => t('Connection information'),
    '#description' => t('Information to connect to EnterMedia server.'),
    '#collapsible' => TRUE,
    '#weight' => 0,
  );
  $form['server_info']['embridge_server_url'] = array(
    '#type' => 'textfield',
    '#title' => t('Hostname'),
    '#size' => 100,
    '#required' => TRUE,
    '#description' => t('EnterMedia Hostname (e.g. http://demo.entermediasoftware.com).'),
    '#default_value' => variable_get('embridge_server_url', 'http://demo.entermediasoftware.com'),
  );
  $form['server_info']['embridge_server_port'] = array(
    '#type' => 'textfield',
    '#title' => t('Port'),
    '#size' => 7,
    '#required' => TRUE,
    '#description' => t('EnterMedia server port (e.g. 8080).'),
    '#default_value' => variable_get('embridge_server_port', ''),
  );
//  $form['server_info']['embridge_server_asset_path'] = array(
//    '#type' => 'textfield',
//    '#title' => t('Asset Path Patterns'),
//    '#size' => 100,
//    '#description' => t('Used to create subfolders during bulk upload. Use token [dam:yyyy] for current year and [dam:mm] for current month.'),
//    '#default_value' => variable_get('embridge_server_asset_path', '[dam:yyyy]/[dam:mm]'),
//  );

  // User information.
  $form['user_info'] = array(
    '#type' => 'fieldset',
    '#title' => t('User information'),
    '#description' => t('Login and password for EnterMedia service.'),
    '#collapsible' => TRUE,
    '#weight' => 5,
  );
  $form['user_info']['embridge_login'] = array(
    '#type' => 'textfield',
    '#title' => t('Login'),
    '#required' => TRUE,
    '#description' => t('Login for EnterMedia service.'),
    '#default_value' => variable_get('embridge_login', 'admin'),
  );
  $form['user_info']['embridge_password'] = array(
    '#type' => 'password',
    '#title' => t('Password'),
    '#required' => TRUE,
    '#description' => t('Password for EnterMedia service.'),
    '#attributes' => array('value' => variable_get('embridge_password', 'admin')),
  );

  $form['test_connection'] = array(
    '#type' => 'button',
    '#value' => t('Test Connection'),
    '#description' => t('Click to test connection using above information.'),
    '#weight' => 10,
  );

  // User information.
  $form['search_info'] = array(
    '#type' => 'fieldset',
    '#title' => t('Search information'),
    '#description' => t('Settings for EnterMedia asset search.'),
    '#collapsible' => TRUE,
    '#weight' => 15,
  );
  $form['search_info']['embridge_search_pagesize'] = array(
    '#type' => 'textfield',
    '#title' => t('Page Size'),
    '#description' => t('Page size for EnterMedia search result.'),
    '#default_value' => variable_get('embridge_search_pagesize', 9),
  );

  embridge_catalog_settings($form);
  
  $form['add_catalog'] = array(
    '#type' => 'fieldset',
    '#title' => t('Add Catalog'),
    '#collapsible' => FALSE,
    '#weight' => 25,
  );
  $form['add_catalog']['new_catalog_id'] = array(
    '#type' => 'textfield',
    '#title' => t('ID'),
    '#description' => t('Enter Catalog ID (e.g. media/catalogs/photo).'),
  );
  
  $form['#validate'][] = '_embridge_settings_validate';
  $form['#submit'][] = '_embridge_settings_submit';

  return system_settings_form($form);
}

/**
 * EnterMedia Catalog Settings.
 */
function embridge_catalog_settings(&$form) {
  global $base_url;
  $catalogs = variable_get('embridge_catalogs', array());
  foreach ($catalogs as $catalog_id => $catalog_name) {
    $form[$catalog_name] = array(
      '#type' => 'fieldset',
      '#title' => $catalog_name . ' Catalog',
      '#description' => t('Search settings in @catalog_name Catalog', array('@catalog_name' => $catalog_name)),
      '#collapsible' => TRUE,
      '#prefix' => '<div id="catalog-' . $catalog_name .'-wrapper" >',
      '#suffix' => '</div>',
      '#weight' => 20,
    );
    
    $form[$catalog_name]['embridge_' . $catalog_name . '_remove'] = array(
      '#type' => 'checkbox',
      '#title' => t('Remove Catalog'),
      '#weight' => 1,
    );

    $form[$catalog_name]['embridge_' . $catalog_name . '_hot_folder'] = array(
      '#type' => 'textfield',
      '#title' => t('Hot Folder'),
      '#size' => 20,
      '#required' => TRUE,
      '#description' => t('EnterMedia hot folder for the catalog (e.g. assets).'),
      '#default_value' => variable_get('embridge_' . $catalog_name . '_hot_folder', ''),
      '#weight' => 2,
    );
    $form[$catalog_name]['embridge_' . $catalog_name . '_server_mediastore'] = array(
      '#type' => 'textfield',
      '#title' => t('Media store path'),
      '#size' => 100,
      '#required' => TRUE,
      '#description' => t('EnterMedia Media store path for the catalog (e.g. /media/assets).'),
      '#default_value' => variable_get('embridge_' . $catalog_name . '_server_mediastore', ''),
      '#weight' => 2,
    );
    
    $form[$catalog_name]['embridge_' . $catalog_name . '_application_id'] = array(
      '#type' => 'textfield',
      '#title' => t('Application ID'),
      '#size' => 100,
      '#description' => t('Application ID for the catalog.'),
      '#default_value' => variable_get('embridge_' . $catalog_name . '_application_id', ''),
      '#weight' => 3,
    );
    
    $fields = variable_get('embridge_' . $catalog_name . '_fields', array());
    
    $form[$catalog_name]['embridge_' . $catalog_name . '_fields'] = array(
      '#type' => 'item',
      '#title' => t('Fields'),
      '#weight' => 4,
      '#tree' => TRUE,
      '#theme' => 'embridge_fields_table',
    );
    
    $data_types = array('' => '', 'text' => 'Text', 'number' => 'Number', 'date' => 'Date', 'list' => 'List');
    
    foreach ($fields as $field_id => $field) {
      $form[$catalog_name]['embridge_' . $catalog_name . '_fields'][$field_id] = array(
        'label' => array(
          '#markup' => $field_id
         ),
        'id' => array(
          '#type' => 'hidden',
          '#value' => $field_id
         ),
        'name' => array(
          '#type' => 'textfield',
          '#default_value' => empty($field['name']) ? '' : $field['name'],
         ),
         'datatype' => array(
          '#type' => 'select',
          '#options' => $data_types, 
          '#default_value' => empty($field['datatype']) ? '' : $field['datatype'],
         ),
         'include_in_search' => array(
          '#type' => 'checkbox',
          '#default_value' => empty($field['include_in_search']) ? 0 : $field['include_in_search'],
         ),
        'include_in_result' => array(
          '#type' => 'checkbox',
          '#default_value' => empty($field['include_in_result']) ? 0 : $field['include_in_result'],
         ),
        'updatable' => array(
          '#type' => 'checkbox',
          '#default_value' => empty($field['updatable']) ? 0 : $field['updatable'],
         ),
        'remove' => array(
          '#type' => 'checkbox',
         ),
      );
    }
    
    $form[$catalog_name]['embridge_' . $catalog_name . '_new_field_id'] = array(
      '#type' => 'textfield',
      '#title' => t('Add Field'),
      '#description' => t('Add EnterMedia Field ID (e.g. name).'),
      '#weight' => 5,
    );

    $renditions = variable_get('embridge_' . $catalog_name . '_renditions', array());
    
    $form[$catalog_name]['embridge_' . $catalog_name . '_renditions'] = array(
      '#type' => 'item',
      '#title' => t('Conversions'),
      '#weight' => 6,
      '#tree' => TRUE,
      '#theme' => 'embridge_renditions_table',
    );

    foreach ($renditions as $rendition_id => $rendition) {
      $form[$catalog_name]['embridge_' . $catalog_name . '_renditions'][$rendition_id] = array(
        'selected' => array(
          '#type' => 'checkbox',
          '#default_value' => empty($rendition['selected']) ? 0 : $rendition['selected'],
         ),
        'id' => array(
          '#markup' => $rendition_id
         ),
        'filename' => array(
          '#type' => 'textfield',
          '#default_value' => empty($rendition['filename']) ? '' : $rendition['filename'],
         ),
         'system' => array(
          '#type' => 'checkbox',
          '#default_value' => empty($rendition['system']) ? 0 : $rendition['system'],
         ),
        'remove' => array(
          '#type' => 'checkbox',
         ),
      );
    }
    
    $form[$catalog_name]['embridge_' . $catalog_name . '_new_rendition_id'] = array(
      '#type' => 'textfield',
      '#title' => t('Add Conversion'),
      '#description' => t('Add EnterMedia Conversion ID (e.g. thumb).'),
      '#weight' => 7,
    );
      
    $form[$catalog_name]['embridge_' . $catalog_name . '_submit'] = array(
      '#type' => 'submit',
      '#value' => t('Save ' . $catalog_name . ' Catalog'),
      '#weight' => 15,
//      '#ajax' => array(
//        'callback' => '_embridge_catalog_settings_submit_callback',
//        'wrapper' => 'catalog-' . $catalog_name .'-wrapper',
//        'method' => 'replace',
//        'effect' => 'fade',
//      ),
    );
  }
}

/**
 * Extra validation for embridge settings.
 */
function _embridge_settings_validate($form, &$form_state) {
  // Validate URL.
  if (!valid_url($form_state['values']['embridge_server_url'], TRUE)) {
    form_set_error('embridge_server_url', t('Invalid URL.'));
  }
  // Validate Port.
  if (!is_numeric($form_state['values']['embridge_server_port'])) {
    form_set_error('embridge_server_port', t('Invalid Port.'));
  }
  
  if (!empty($form_state['values']['new_catalog_id']) 
      && strpos($form_state['values']['new_catalog_id'], 'media/catalogs/') !==0) {
    form_set_error('new_catalog_id', t('Invalid Catalog ID.'));
  }
}

/**
 * Extra validation for embridge settings.
 */
function _embridge_settings_submit($form, &$form_state) {
  $catalogs = variable_get('embridge_catalogs', array());
  foreach ($catalogs as $catalog_id => $catalog_name) {
    // Remove existing catalog.
    if (!empty($form_state['values']['embridge_' . $catalog_name . '_remove'])) {
      unset($catalogs[$catalog_id]);
      // Clean variables related to the catalog.
      db_delete('variable')
        ->condition('name', db_like('embridge_' . $catalog_name . '_') . '%', 'like')
        ->execute();
      foreach($form_state['values'] as $key => $value) {
        if (strpos($key, 'embridge_' . $catalog_name . '_') === 0) {
          unset($form_state['values'][$key]);
        }
      }
    }
    else {
      if (!empty($form_state['values']['embridge_' . $catalog_name . '_renditions'])) {
        foreach($form_state['values']['embridge_' . $catalog_name . '_renditions'] as $rendtion_id => $rendtion) {
          if (!empty($rendtion['remove'])) {
            unset($form_state['values']['embridge_' . $catalog_name . '_renditions'][$rendtion_id]);
          }
        }
      }
      if (!empty($form_state['values']['embridge_' . $catalog_name . '_new_rendition_id'])) {
        $rendtion_id = $form_state['values']['embridge_' . $catalog_name . '_new_rendition_id'];
        $form_state['values']['embridge_' . $catalog_name . '_renditions'][$rendtion_id] = array(
          'selected' => FALSE,
          'filename' => '',
          'remove' => FALSE,
        );
      }
      unset($form_state['values']['embridge_' . $catalog_name . '_new_rendition_id']);
      
      if (!empty($form_state['values']['embridge_' . $catalog_name . '_fields'])) {
        foreach($form_state['values']['embridge_' . $catalog_name . '_fields'] as $rendtion_id => $rendtion) {
          if (!empty($rendtion['remove'])) {
            unset($form_state['values']['embridge_' . $catalog_name . '_fields'][$rendtion_id]);
          }
        }
      }
      if (!empty($form_state['values']['embridge_' . $catalog_name . '_new_field_id'])) {
        $field_id = $form_state['values']['embridge_' . $catalog_name . '_new_field_id'];
        $form_state['values']['embridge_' . $catalog_name . '_fields'][$field_id] = array(
          'id' =>$form_state['values']['embridge_' . $catalog_name . '_new_field_id'],
          'name' => '',
          'datatype' => '',
          'include_in_search' => FALSE,
          'include_in_result' => FALSE,
          'updatable' => FALSE,
          'remove' => FALSE,
        );
      }
      unset($form_state['values']['embridge_' . $catalog_name . '_new_field_id']);
    }
  }
  
  // Add new catalog.
  if (!empty($form_state['values']['new_catalog_id'])) {
    $catalog_name = array_pop(explode('/', $form_state['values']['new_catalog_id']));
    $catalogs[$form_state['values']['new_catalog_id']] = $catalog_name;
  }
  
  unset($form_state['values']['new_catalog_id']);
  variable_set('embridge_catalogs', $catalogs);
}

function _embridge_catalog_settings_submit_callback($form, &$form_state) {
  $catalog_name = current($form_state['clicked_button']['#array_parents']);
  $form_state['rebuild'] = TRUE;
  $form = drupal_rebuild_form($form['#form_id'], $form_state);
  $form[$catalog_name]['embridge_' . $catalog_name . '_new_rendition_id']['#value'] = '';
  $form[$catalog_name]['embridge_' . $catalog_name . '_new_field_id']['#value'] = ''; 
  return $form[$catalog_name];
}

/**
 * Test connection for the EnterMedia Server.
 *
 * @return string
 *   JSON indicate status of the connection.
 */
function embridge_test_connection() {
  $server_url = filter_xss($_POST['server']);
  $server_port = filter_xss($_POST['port']);
  $login = filter_xss($_POST['login']);
  $password = filter_xss($_POST['password']);
  $login_url = $server_url . ':' . $server_port . EMBRIDGE_LOGIN_PATH_DEFAULT . '?catalogid=media&accountname=' . $login . '&password=' . $password;
  watchdog('embridge', '%login_url called. ', array('%login_url' => $login_url), WATCHDOG_NOTICE);
  $xml = _embridge_curl_grab_page($server_url . ':' . $server_port . EMBRIDGE_LOGIN_PATH_DEFAULT, EMBRIDGE_QUERY_REFERER_DEFAULT, '&catalogid=media&accountname=' . $login . '&password=' . $password, 'true', 'null');
  watchdog('embridge', '%xml called. ', array('%xml' => $xml), WATCHDOG_NOTICE);
  $xml_obj = simplexml_load_string($xml);
  $xml_arr = (array) $xml_obj;
  if ($xml_arr['@attributes']['stat'] == 'ok') {
    print drupal_json_output(array('status' => t('Connected to EnterMedia server successfully.')));
  }
  else {
    print drupal_json_output(array('status' => t('Failed to EnterMedia server using above information.')));
  }
  exit();
}


function theme_embridge_renditions_table($variables) {
  $element = $variables['element'];
  $headers = array(t('Selected'), t('ID'), t('File Name'), t('System'), t('Remove'));
  $rows = array();
  foreach (element_children($element) as $rendition_id) {
    $row = array(
      'data' => array(
        drupal_render($element[$rendition_id]['selected']),
        drupal_render($element[$rendition_id]['id']),
        drupal_render($element[$rendition_id]['filename']),
        drupal_render($element[$rendition_id]['system']),
        drupal_render($element[$rendition_id]['remove']),
      ), 
    );
    $rows[] = $row;
  }
  $output = '';
  if (!empty($rows)) {
    $output .= theme('table', array(
      'header' => $headers, 
      'rows' => $rows,
      'attributes' => array('class' => array('renditions-table')),
    ));
  }
  return $output;
}

function theme_embridge_fields_table($variables) {
  $element = $variables['element'];
  $headers = array(t('ID'), t('Field Name'), t('Type'), t('Searchable'), t('Include in Result'), t('Editable'), t('Remove'));
  $rows = array();
  foreach (element_children($element) as $field_id) {
    $row = array(
      'data' => array(
        drupal_render($element[$field_id]['label']) . drupal_render($element[$field_id]['id']),
        drupal_render($element[$field_id]['name']),
        drupal_render($element[$field_id]['datatype']),
        drupal_render($element[$field_id]['include_in_search']),
        drupal_render($element[$field_id]['include_in_result']),
        drupal_render($element[$field_id]['updatable']),
        drupal_render($element[$field_id]['remove']),
      ), 
    );
    $rows[] = $row;
  }
  $output = '';
  if (!empty($rows)) {
    $output .= theme('table', array(
      'header' => $headers, 
      'rows' => $rows,
      'attributes' => array('class' => array('fields-table')),
    ));
  }
  return $output;
}